@@ -1,7 +1,7 @@
 /*
  * opngreduc.c - libpng extension: lossless image reductions.
  *
- * Copyright (C) 2003-2008 Cosmin Truta.
+ * Copyright (C) 2003-2011 Cosmin Truta.
  * This software is distributed under the same licensing and warranty terms
  * as libpng.
  *
@@ -19,17 +19,28 @@
  * in the implicit form, without any transformation in effect.
  */
 
-
-/* The access to the libpng internals is obtained by
- * #defining PNG_INTERNAL (for libpng 1.2.x or older) or
- * #including "pngpriv.h" (for libpng 1.4.x or newer).
- */
 #define PNG_INTERNAL
+#define PNG_NO_PEDANTIC_WARNINGS
 #include "png.h"
+#include "opngreduc.h"
+
 #if PNG_LIBPNG_VER >= 10400
+#ifdef PNG_USER_PRIVATEBUILD
 #include "pngpriv.h"
 #endif
-#include "opngreduc.h"
+#endif
+
+#ifndef OPNG_ASSERT
+#include <assert.h>
+#define OPNG_ASSERT(cond) assert(cond)
+#define OPNG_ASSERT_MSG(cond, msg) assert(cond)
+#endif
+
+#ifdef png_debug
+#define opng_debug(level, msg) png_debug(level, msg)
+#else
+#define opng_debug(level, msg) ((void)0)
+#endif
 
 
 #ifdef OPNG_IMAGE_REDUCTIONS_SUPPORTED
@@ -45,34 +56,45 @@ __error__ "OPNG_IMAGE_REDUCTIONS_SUPPORTED" requires support for bKGD,hIST,sBIT,
 #endif
 
 
-#if !(PNG_LIBPNG_BUILD_TYPE & PNG_LIBPNG_BUILD_PRIVATE)
+/*****************************************************************************
+ * Quick and dirty libpng hacks - don't look ;-)
+ *****************************************************************************/
+
+#if PNG_LIBPNG_VER < 10400
+#define trans_alpha trans
+#define trans_color trans_values
+#endif
+
+#define OPNG_FILLER 0x8000L
+#ifdef PNG_FILLER
+#if PNG_FILLER != OPNG_FILLER
+__error__ "PNG_FILLER" has an unexpected value
+#endif
+#endif
+
+#define OPNG_FLAG_FILLER_AFTER 0x0080
+#ifdef PNG_FLAG_FILLER_AFTER
+#if PNG_FLAG_FILLER_AFTER != OPNG_FLAG_FILLER_AFTER
+__error__ "PNG_FLAG_FILLER_AFTER" has an unexpected value
+#endif
+#endif
+
 /*
- * First and foremost, make sure that direct access to libpng's
- * internal structures does not break the consistency of data.
- * (This quick-and-dirty function should go away when opngreduc.c
- * is incorporated into libpng.)
+ * Check if it's safe to access libpng's internal structures directly.
+ * Some fields might have their offsets shifted due to changes in
+ * libpng's configuration.
  */
 static void
 _opng_validate_internal(png_structp png_ptr, png_infop info_ptr)
 {
-   png_uint_32 width, height;
-   int bit_depth, color_type;
    png_color_16p background;
    png_uint_16p hist;
    png_color_8p sig_bit;
-   png_bytep trans;
+   png_bytep trans_alpha;
    int num_trans;
-   png_color_16p trans_values;
+   png_color_16p trans_color;
 
    /* Check info_ptr. */
-   if (png_get_valid(png_ptr, info_ptr, (png_uint_32)-1) != info_ptr->valid)
-      goto error;
-   png_get_IHDR(png_ptr, info_ptr,
-      &width, &height, &bit_depth, &color_type, NULL, NULL, NULL);
-   /* This particular check should never fail, but we're doing it anyway. */
-   if (width != info_ptr->width || height != info_ptr->height ||
-       bit_depth != info_ptr->bit_depth || color_type != info_ptr->color_type)
-      goto error;
    if (png_get_rows(png_ptr, info_ptr) != info_ptr->row_pointers)
       goto error;
    if (png_get_bKGD(png_ptr, info_ptr, &background))
@@ -84,13 +106,13 @@ _opng_validate_internal(png_structp png_ptr, png_infop info_ptr)
    if (png_get_sBIT(png_ptr, info_ptr, &sig_bit))
       if (sig_bit != &info_ptr->sig_bit)
          goto error;
-   if (png_get_tRNS(png_ptr, info_ptr, &trans, &num_trans, &trans_values))
-      if ((trans != NULL &&
-           (trans != info_ptr->trans || num_trans != info_ptr->num_trans)) ||
-          (trans_values != NULL && trans_values != &info_ptr->trans_values))
+   if (png_get_tRNS(png_ptr, info_ptr, &trans_alpha, &num_trans, &trans_color))
+      if ((trans_alpha != NULL && (trans_alpha != info_ptr->trans_alpha ||
+                                   num_trans != info_ptr->num_trans)) ||
+          (trans_color != NULL && trans_color != &info_ptr->trans_color))
          goto error;
 
-   /* Also check png_ptr.  (It's not much, but we're doing what we can.) */
+   /* Also check png_ptr. It's not much, we're doing what we can... */
    if (png_get_compression_buffer_size(png_ptr) != png_ptr->zbuf_size)
       goto error;
 
@@ -101,42 +123,40 @@ error:
    png_error(png_ptr,
       "[internal error] Inconsistent internal structures (incorrect libpng?)");
 }
-#endif
 
 
+/*****************************************************************************
+ * To be added to png.c
+ *****************************************************************************/
+
 /*
- * Indicate whether the image information is valid.
- * (The image information is valid if all the required critical
- * information is present in the png structures.)
+ * Check if the image information is valid.
+ * The image information is said to be valid if all the required
+ * critical information is present in the png structures.
  * The function returns 1 if this info is valid, and 0 otherwise.
- * If there is some inconsistency in the internal structures
- * (due to incorrect libpng use, or to libpng bugs), the function
- * issues a png_error.
+ * If there is any inconsistency in the internal structures
+ * (possibly caused by incorrect libpng use, or by libpng bugs),
+ * the function issues a png_error.
  */
 int PNGAPI
 opng_validate_image(png_structp png_ptr, png_infop info_ptr)
 {
-   int result = 1, error = 0;
+   int result, error;
 
-   png_debug(1, "in opng_validate_image\n");
+   opng_debug(1, "in opng_validate_image");
 
    if (png_ptr == NULL || info_ptr == NULL)
       return 0;
 
-#if !(PNG_LIBPNG_BUILD_TYPE & PNG_LIBPNG_BUILD_PRIVATE)
    _opng_validate_internal(png_ptr, info_ptr);
-#endif
+
+   result = 1;
+   error = 0;
 
    /* Validate IHDR. */
-   /* We cannot use png_get_IHDR() because it issues an error. */
-   if (info_ptr->width > 0 && info_ptr->height > 0)
+   if (info_ptr->bit_depth != 0)
    {
-      if (info_ptr->width > PNG_UINT_31_MAX ||
-          info_ptr->height > PNG_UINT_31_MAX ||
-          info_ptr->bit_depth == 0 ||
-          info_ptr->bit_depth > 16 ||
-          info_ptr->interlace_type >= PNG_INTERLACE_LAST)
-          /* need more checks... */
+      if (info_ptr->width == 0 || info_ptr->height == 0)
          error = 1;
    }
    else
@@ -186,6 +206,10 @@ opng_validate_image(png_structp png_ptr, png_infop info_ptr)
             ((int)(B1) - (int)(B2)))))
 
 
+/*****************************************************************************
+ * To be added to pngreduc.c
+ *****************************************************************************/
+
 /*
  * Build a color+alpha palette in which the entries are sorted by
  * (alpha, red, green, blue), in this particular order.
@@ -198,7 +222,7 @@ opng_validate_image(png_structp png_ptr, png_infop info_ptr)
  */
 static int /* PRIVATE */
 opng_insert_palette_entry(png_colorp palette, int *num_palette,
-   png_bytep trans, int *num_trans, int max_tuples,
+   png_bytep trans_alpha, int *num_trans, int max_tuples,
    unsigned int red, unsigned int green, unsigned int blue, unsigned int alpha,
    int *index)
 {
@@ -216,7 +240,7 @@ opng_insert_palette_entry(png_colorp palette, int *num_palette,
       {
          mid = (low + high) / 2;
          cmp = OPNG_CMP_ALPHA_COLOR(alpha, red, green, blue,
-            trans[mid],
+            trans_alpha[mid],
             palette[mid].red, palette[mid].green, palette[mid].blue);
          if (cmp < 0)
             high = mid - 1;
@@ -285,8 +309,8 @@ opng_insert_palette_entry(png_colorp palette, int *num_palette,
    {
       OPNG_ASSERT(low <= *num_trans);
       for (i = *num_trans; i > low; --i)
-         trans[i] = trans[i - 1];
-      trans[low] = (png_byte)alpha;
+         trans_alpha[i] = trans_alpha[i - 1];
+      trans_alpha[low] = (png_byte)alpha;
       ++(*num_trans);
    }
    *index = low;
@@ -304,7 +328,7 @@ opng_get_alpha_row(png_structp png_ptr, png_infop info_ptr,
    png_bytep sample_ptr;
    png_uint_32 width, i;
    unsigned int channels;
-   png_color_16p trans_values;
+   png_color_16p trans_color;
 
    OPNG_ASSERT(info_ptr->bit_depth == 8);
    OPNG_ASSERT(!(info_ptr->color_type & PNG_COLOR_MASK_PALETTE));
@@ -317,12 +341,12 @@ opng_get_alpha_row(png_structp png_ptr, png_infop info_ptr,
          memset(alpha_row, 255, (size_t)width);
          return;
       }
-      trans_values = &info_ptr->trans_values;
+      trans_color = &info_ptr->trans_color;
       if (info_ptr->color_type == PNG_COLOR_TYPE_RGB)
       {
-         png_byte trans_red   = (png_byte)trans_values->red;
-         png_byte trans_green = (png_byte)trans_values->green;
-         png_byte trans_blue  = (png_byte)trans_values->blue;
+         png_byte trans_red   = (png_byte)trans_color->red;
+         png_byte trans_green = (png_byte)trans_color->green;
+         png_byte trans_blue  = (png_byte)trans_color->blue;
          for (i = 0; i < width; ++i)
             alpha_row[i] = (png_byte)
                ((row[3*i] == trans_red && row[3*i+1] == trans_green &&
@@ -330,7 +354,7 @@ opng_get_alpha_row(png_structp png_ptr, png_infop info_ptr,
       }
       else
       {
-         png_byte trans_gray = (png_byte)trans_values->gray;
+         png_byte trans_gray = (png_byte)trans_color->gray;
          OPNG_ASSERT(info_ptr->color_type == PNG_COLOR_TYPE_GRAY);
          for (i = 0; i < width; ++i)
             alpha_row[i] = (png_byte)(row[i] == trans_gray ? 0 : 255);
@@ -366,7 +390,7 @@ opng_analyze_bits(png_structp png_ptr, png_infop info_ptr,
       offset_color, offset_alpha;
    png_color_16p background;
 
-   png_debug(1, "in opng_analyze_bits\n");
+   opng_debug(1, "in opng_analyze_bits");
 
    bit_depth = info_ptr->bit_depth;
    if (bit_depth < 8)
@@ -526,13 +550,17 @@ opng_reduce_bits(png_structp png_ptr, png_infop info_ptr,
    png_bytepp row_ptr;
    png_bytep src_ptr, dest_ptr;
    png_uint_32 height, width, i, j;
-   unsigned int
-      src_bit_depth, dest_bit_depth, src_byte_depth, dest_byte_depth,
-      src_color_type, dest_color_type, src_channels, dest_channels,
-      src_sample_size, dest_sample_size, src_offset_alpha;
-   unsigned int tran_tbl[8], k;
+   unsigned int src_bit_depth, dest_bit_depth;
+   unsigned int src_byte_depth, dest_byte_depth;
+   unsigned int src_color_type, dest_color_type;
+   unsigned int src_channels, dest_channels;
+   unsigned int src_sample_size, dest_sample_size;
+   unsigned int dest_pixel_depth;
+   unsigned int src_offset_alpha;
+   unsigned int tran_tbl[8];
+   unsigned int k;
 
-   png_debug(1, "in opng_reduce_bits\n");
+   opng_debug(1, "in opng_reduce_bits");
 
    /* See which reductions may be performed. */
    reductions = opng_analyze_bits(png_ptr, info_ptr, reductions);
@@ -576,7 +604,7 @@ opng_reduce_bits(png_structp png_ptr, png_infop info_ptr,
 
    src_sample_size  = src_channels * src_byte_depth;
    dest_sample_size = dest_channels * dest_byte_depth;
-   OPNG_ASSERT(src_sample_size > dest_sample_size);
+   dest_pixel_depth = dest_channels * dest_bit_depth;
 
    if (!(png_ptr->transformations & PNG_FILLER) ||
        (png_ptr->flags & PNG_FLAG_FILLER_AFTER))
@@ -620,6 +648,7 @@ opng_reduce_bits(png_structp png_ptr, png_infop info_ptr,
    }
 
    /* Translate the samples to the new image type. */
+   OPNG_ASSERT(src_sample_size > dest_sample_size);
    row_ptr = info_ptr->row_pointers;
    height  = info_ptr->height;
    width   = info_ptr->width;
@@ -677,18 +706,18 @@ opng_reduce_bits(png_structp png_ptr, png_infop info_ptr,
    }
    if (info_ptr->valid & PNG_INFO_tRNS)
    {
-      png_color_16p trans_values = &info_ptr->trans_values;
+      png_color_16p trans_color = &info_ptr->trans_color;
       if (reductions & OPNG_REDUCE_16_TO_8)
       {
-         if (trans_values->red   % 257 == 0 &&
-             trans_values->green % 257 == 0 &&
-             trans_values->blue  % 257 == 0 &&
-             trans_values->gray  % 257 == 0)
+         if (trans_color->red   % 257 == 0 &&
+             trans_color->green % 257 == 0 &&
+             trans_color->blue  % 257 == 0 &&
+             trans_color->gray  % 257 == 0)
          {
-            trans_values->red   &= 255;
-            trans_values->green &= 255;
-            trans_values->blue  &= 255;
-            trans_values->gray  &= 255;
+            trans_color->red   &= 255;
+            trans_color->green &= 255;
+            trans_color->blue  &= 255;
+            trans_color->gray  &= 255;
          }
          else
          {
@@ -699,9 +728,9 @@ opng_reduce_bits(png_structp png_ptr, png_infop info_ptr,
       }
       if (reductions & OPNG_REDUCE_RGB_TO_GRAY)
       {
-         if (trans_values->red == trans_values->green ||
-             trans_values->red == trans_values->blue)
-            trans_values->gray = trans_values->red;
+         if (trans_color->red == trans_color->green ||
+             trans_color->red == trans_color->blue)
+            trans_color->gray = trans_color->red;
          else
          {
             /* Non-gray tRNS in grayscale image: all pixels are 100% opaque. */
@@ -712,11 +741,11 @@ opng_reduce_bits(png_structp png_ptr, png_infop info_ptr,
    }
 
    /* Update the image info. */
+   png_ptr->rowbytes    = info_ptr->rowbytes    = 0;
    png_ptr->bit_depth   = info_ptr->bit_depth   = (png_byte)dest_bit_depth;
    png_ptr->color_type  = info_ptr->color_type  = (png_byte)dest_color_type;
    png_ptr->channels    = info_ptr->channels    = (png_byte)dest_channels;
-   png_ptr->pixel_depth = info_ptr->pixel_depth =
-      (png_byte)(dest_bit_depth * dest_channels);
+   png_ptr->pixel_depth = info_ptr->pixel_depth = (png_byte)dest_pixel_depth;
    if (reductions & OPNG_REDUCE_STRIP_ALPHA)
    {
       png_ptr->transformations &= ~PNG_FILLER;
@@ -744,7 +773,7 @@ opng_reduce_palette_bits(png_structp png_ptr, png_infop info_ptr,
    unsigned int src_mask_init, src_mask, src_shift, dest_shift;
    unsigned int sample, dest_buf;
 
-   png_debug(1, "in opng_reduce_palette_bits\n");
+   opng_debug(1, "in opng_reduce_palette_bits");
 
    /* Check if the reduction applies. */
    if (!(reductions & OPNG_REDUCE_8_TO_4_2_1) ||
@@ -834,7 +863,8 @@ opng_reduce_palette_bits(png_structp png_ptr, png_infop info_ptr,
    }
 
    /* Update the image info. */
-   png_ptr->bit_depth   = info_ptr->bit_depth   =
+   png_ptr->rowbytes    = info_ptr->rowbytes    = 0;
+   png_ptr->bit_depth   = info_ptr->bit_depth   = (png_byte)dest_bit_depth;
    png_ptr->pixel_depth = info_ptr->pixel_depth = (png_byte)dest_bit_depth;
 
    return OPNG_REDUCE_8_TO_4_2_1;
@@ -857,13 +887,13 @@ opng_reduce_to_palette(png_structp png_ptr, png_infop info_ptr,
    png_uint_32 height, width, channels, i, j;
    unsigned int color_type, dest_bit_depth;
    png_color palette[256];
-   png_byte trans[256];
+   png_byte trans_alpha[256];
    int num_palette, num_trans, index;
    png_color_16p background;
    unsigned int gray, red, green, blue, alpha;
    unsigned int prev_gray, prev_red, prev_green, prev_blue, prev_alpha;
 
-   png_debug(1, "in opng_reduce_to_palette\n");
+   opng_debug(1, "in opng_reduce_to_palette");
 
    if (info_ptr->bit_depth != 8)
       return OPNG_REDUCE_NONE;  /* nothing is done in this case */
@@ -901,7 +931,7 @@ opng_reduce_to_palette(png_structp png_ptr, png_infop info_ptr,
                prev_blue  = blue;
                prev_alpha = alpha;
                if (opng_insert_palette_entry(palette, &num_palette,
-                   trans, &num_trans, 256,
+                   trans_alpha, &num_trans, 256,
                    red, green, blue, alpha, &index) < 0)  /* overflow */
                {
                   OPNG_ASSERT(num_palette < 0);
@@ -923,7 +953,7 @@ opng_reduce_to_palette(png_structp png_ptr, png_infop info_ptr,
                prev_gray  = gray;
                prev_alpha = alpha;
                if (opng_insert_palette_entry(palette, &num_palette,
-                   trans, &num_trans, 256,
+                   trans_alpha, &num_trans, 256,
                    gray, gray, gray, alpha, &index) < 0)  /* overflow */
                {
                   OPNG_ASSERT(num_palette < 0);
@@ -946,7 +976,8 @@ opng_reduce_to_palette(png_structp png_ptr, png_infop info_ptr,
       }
       else
          red = green = blue = background->gray;
-      opng_insert_palette_entry(palette, &num_palette, trans, &num_trans, 256,
+      opng_insert_palette_entry(palette, &num_palette,
+         trans_alpha, &num_trans, 256,
          red, green, blue, 256, &index);
       if (index >= 0)
          background->index = (png_byte)index;
@@ -1011,7 +1042,7 @@ opng_reduce_to_palette(png_structp png_ptr, png_infop info_ptr,
                prev_blue  = blue;
                prev_alpha = alpha;
                if (opng_insert_palette_entry(palette, &num_palette,
-                   trans, &num_trans, 256,
+                   trans_alpha, &num_trans, 256,
                    red, green, blue, alpha, &index) != 0)
                   index = -1;  /* this should not happen */
             }
@@ -1031,7 +1062,7 @@ opng_reduce_to_palette(png_structp png_ptr, png_infop info_ptr,
                prev_gray  = gray;
                prev_alpha = alpha;
                if (opng_insert_palette_entry(palette, &num_palette,
-                   trans, &num_trans, 256,
+                   trans_alpha, &num_trans, 256,
                    gray, gray, gray, alpha, &index) != 0)
                   index = -1;  /* this should not happen */
             }
@@ -1042,12 +1073,13 @@ opng_reduce_to_palette(png_structp png_ptr, png_infop info_ptr,
    }
 
    /* Update the image info. */
+   png_ptr->rowbytes    = info_ptr->rowbytes    = 0;
    png_ptr->color_type  = info_ptr->color_type  = PNG_COLOR_TYPE_PALETTE;
    png_ptr->channels    = info_ptr->channels    = 1;
    png_ptr->pixel_depth = info_ptr->pixel_depth = 8;
    png_set_PLTE(png_ptr, info_ptr, palette, num_palette);
    if (num_trans > 0)
-      png_set_tRNS(png_ptr, info_ptr, trans, num_trans, NULL);
+      png_set_tRNS(png_ptr, info_ptr, trans_alpha, num_trans, NULL);
    /* bKGD (if present) is already updated. */
 
    png_free(png_ptr, alpha_row);
@@ -1074,7 +1106,7 @@ opng_analyze_sample_usage(png_structp png_ptr, png_infop info_ptr,
    png_uint_32 width, height, i, j;
    unsigned int bit_depth, init_shift, init_mask, shift, mask;
 
-   png_debug(1, "in opng_analyze_sample_usage\n");
+   opng_debug(1, "in opng_analyze_sample_usage");
 
    row_ptr = info_ptr->row_pointers;
    height  = info_ptr->height;
@@ -1131,7 +1163,7 @@ opng_reduce_palette(png_structp png_ptr, png_infop info_ptr,
 {
    png_uint_32 result;
    png_colorp palette;
-   png_bytep trans;
+   png_bytep trans_alpha;
    png_bytepp rows;
    png_uint_32 width, height, i, j;
    png_byte is_used[256];
@@ -1139,7 +1171,7 @@ opng_reduce_palette(png_structp png_ptr, png_infop info_ptr,
    png_color_16 gray_trans;
    png_byte crt_trans_value, last_trans_value;
 
-   png_debug(1, "in opng_reduce_palette\n");
+   opng_debug(1, "in opng_reduce_palette");
 
    height      = info_ptr->height;
    width       = info_ptr->width;
@@ -1148,14 +1180,14 @@ opng_reduce_palette(png_structp png_ptr, png_infop info_ptr,
    rows        = info_ptr->row_pointers;
    if (info_ptr->valid & PNG_INFO_tRNS)
    {
-      trans     = info_ptr->trans;
-      num_trans = info_ptr->num_trans;
-      OPNG_ASSERT(trans != NULL && num_trans > 0);
+      trans_alpha = info_ptr->trans_alpha;
+      num_trans   = info_ptr->num_trans;
+      OPNG_ASSERT(trans_alpha != NULL && num_trans > 0);
    }
    else
    {
-      trans     = NULL;
-      num_trans = 0;
+      trans_alpha = NULL;
+      num_trans   = 0;
    }
 
    /* Analyze the possible reductions. */
@@ -1170,7 +1202,7 @@ opng_reduce_palette(png_structp png_ptr, png_infop info_ptr,
       if (!is_used[k])
          continue;
       last_color_index = k;
-      if (k < num_trans && trans[k] < 255)
+      if (k < num_trans && trans_alpha[k] < 255)
          last_trans_index = k;
       if (is_gray)
          if (palette[k].red != palette[k].green ||
@@ -1197,14 +1229,14 @@ opng_reduce_palette(png_structp png_ptr, png_infop info_ptr,
    if (is_gray && num_trans > 0)
    {
       gray_trans.gray = palette[last_trans_index].red;
-      last_trans_value = trans[last_trans_index];
+      last_trans_value = trans_alpha[last_trans_index];
       for (k = 0; k <= last_color_index; ++k)
       {
          if (!is_used[k])
             continue;
          if (k <= last_trans_index)
          {
-            crt_trans_value = trans[k];
+            crt_trans_value = trans_alpha[k];
             /* Cannot reduce if different colors have transparency. */
             if (crt_trans_value < 255 && palette[k].red != gray_trans.gray)
             {
@@ -1286,6 +1318,7 @@ opng_reduce_palette(png_structp png_ptr, png_infop info_ptr,
       png_set_tRNS(png_ptr, info_ptr, NULL, 0, &gray_trans);
 
    /* Update the image info. */
+   png_ptr->rowbytes   = info_ptr->rowbytes   = 0;
    png_ptr->color_type = info_ptr->color_type = PNG_COLOR_TYPE_GRAY;
    png_free_data(png_ptr, info_ptr, PNG_FREE_PLTE, -1);
    info_ptr->valid &= ~PNG_INFO_PLTE;
@@ -1308,7 +1341,7 @@ opng_reduce_image(png_structp png_ptr, png_infop info_ptr,
    unsigned int color_type;
    png_uint_32 result;
 
-   png_debug(1, "in opng_reduce_image_type\n");
+   opng_debug(1, "in opng_reduce_image_type");
 
    if (!opng_validate_image(png_ptr, info_ptr))
    {
@@ -1317,16 +1350,6 @@ opng_reduce_image(png_structp png_ptr, png_infop info_ptr,
       return OPNG_REDUCE_NONE;
    }
 
-#if 0  /* PNG_INTERLACE must be recognized! */
-   if (png_ptr->transformations)
-   {
-      png_warning(png_ptr,
-         "Image reduction cannot be applied "
-         "under the presence of transformations");
-      return OPNG_REDUCE_NONE;
-   }
-#endif
-
    color_type = info_ptr->color_type;
 
    /* The reductions below must be applied in the given order. */
