@@ -1,29 +1,30 @@
-/**
- ** OptiPNG: Advanced PNG optimization program.
- ** http://optipng.sourceforge.net/
- **
- ** Copyright (C) 2001-2009 Cosmin Truta.
- **
- ** This software is distributed under the zlib license.
- ** Please see the attached LICENSE for more information.
- **
- ** PNG optimization is described in detail in the PNG-Tech article
- ** "A guide to PNG optimization"
- ** http://www.cs.toronto.edu/~cosmin/pngtech/optipng.html
- **
- ** The idea of running multiple compression trials with different
- ** PNG filters and zlib parameters is inspired from the pngcrush
- ** program by Glenn Randers-Pehrson.
- ** The idea of performing lossless image reductions is inspired from
- ** the pngrewrite program by Jason Summers.
- **
- ** Requirements:
- **    ANSI C89, ISO C90 or ISO C99 compiler and library.
- **    zlib version 1.2.1 or newer.
- **    libpng version 1.2.9 or newer, with pngxtern.
- **    cexcept version 2.0.1 or newer.
- **    POSIX or Windows API for enhanced functionality.
- **/
+/*
+ * OptiPNG: Advanced PNG optimization program.
+ * http://optipng.sourceforge.net/
+ *
+ * Copyright (C) 2001-2010 Cosmin Truta.
+ *
+ * This software is distributed under the zlib license.
+ * Please see the attached LICENSE for more information.
+ *
+ * PNG optimization is described in detail in the PNG-Tech article
+ * "A guide to PNG optimization"
+ * http://optipng.sourceforge.net/pngtech/optipng.html
+ *
+ * The idea of running multiple compression trials with different
+ * PNG filters and zlib parameters is inspired from the pngcrush
+ * program by Glenn Randers-Pehrson.
+ * The idea of performing lossless image reductions is inspired from
+ * the pngrewrite program by Jason Summers.
+ *
+ * Requirements:
+ *    ANSI/ISO C compiler and library.
+ *    zlib version 1.2.1 or newer.
+ *    libpng version 1.2.9 or newer, with pngxtern.
+ *    cexcept version 2.0.1 or newer.
+ *    POSIX or Windows API for enhanced functionality.
+ */
+
 
 #include <ctype.h>
 #include <errno.h>
@@ -32,12 +33,13 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "proginfo.h"
 #include "optipng.h"
+#include "proginfo.h"
+
 #include "cbitset.h"
 #include "osys.h"
-#include "strutil.h"
 #include "png.h"
+#include "pngx.h"
 #include "zlib.h"
 
 
@@ -82,7 +84,7 @@ static const char *msg_help =
    "General options:\n"
    "    -fix\t\tenable error recovery\n"
    "    -force\t\tenforce writing of a new output file\n"
-#if 0  /* parallel processing is not implemented */
+#if 0  /* not implemented */
    "    -jobs <number>\tallow parallel jobs\n"
 #endif
    "    -keep\t\tkeep a backup of the modified files\n"
@@ -90,9 +92,6 @@ static const char *msg_help =
    "    -quiet\t\tquiet mode\n"
    "    -simulate\t\tsimulation mode\n"
    "    -snip\t\tcut one image out of multi-image or animation files\n"
-#if 0  /* multi-image splitting is not implemented */
-   "    -split\t\tsplit multi-image/animation files into separate images\n"
-#endif
    "    -out <file>\t\twrite output file to <file>\n"
    "    -dir <directory>\twrite output file(s) to <directory>\n"
    "    -log <file>\t\tlog messages to <file>\n"
@@ -112,13 +111,14 @@ static const char *msg_help =
    "    -nb\t\t\tno bit depth reduction\n"
    "    -nc\t\t\tno color type reduction\n"
    "    -np\t\t\tno palette reduction\n"
-#if 0  /* metadata optimization is not implemented */
+#if 0  /* not implemented */
    "    -nm\t\t\tno metadata optimization\n"
 #endif
-   "    -nz\t\t\tno IDAT recompression (also disable reductions)\n"
+   "    -nx\t\t\tno reductions\n"
+   "    -nz\t\t\tno IDAT recoding\n"
    "Optimization details:\n"
    "    The optimization level presets\n"
-   "        -o0  <=>  -nz\n"
+   "        -o0  <=>  -o1 -nx -nz\n"
    "        -o1  <=>  [use the libpng heuristics]\t(1 trial)\n"
    "        -o2  <=>  -zc9 -zm8 -zs0-3 -f0,5\t(8 trials)\n"
    "        -o3  <=>  -zc9 -zm8-9 -zs0-3 -f0,5\t(16 trials)\n"
@@ -135,7 +135,7 @@ static const char *msg_help =
    "    optipng file.png\t\t\t\t(default speed)\n"
    "    optipng -o5 file.png\t\t\t(moderately slow)\n"
    "    optipng -o7 file.png\t\t\t(very slow)\n"
-   "    optipng -i1 -o7 -v -full -sim experiment.png -log experiment.log\n";
+   "    optipng -i1 -o7 -v -full -sim experiment.png\n";
 
 
 static enum { OP_NONE, OP_HELP, OP_RUN } operation;
@@ -145,7 +145,9 @@ static FILE *log_file;
 int start_of_line;
 
 
-/** Error handling **/
+/*
+ * Error handling
+ */
 static void
 error(const char *fmt, ...)
 {
@@ -161,81 +163,247 @@ error(const char *fmt, ...)
 }
 
 
-/** Panic handling **/
+/*
+ * Panic handling
+ */
 static void
 panic(const char *msg)
 {
     /* Print the panic message to stderr and terminate abnormally. */
     fprintf(stderr, "\n** INTERNAL ERROR: %s\n", msg);
-    fprintf(stderr, "Please submit a defect report.\n");
-    fprintf(stderr, PROGRAM_URI "\n\n");
+    fprintf(stderr, "Please submit a defect report.\n" PROGRAM_URI "\n\n");
     fflush(stderr);
     osys_terminate();
 }
 
 
-/** String-to-integer conversion **/
-static long
-str2long(const char *str, long *value)
+/*
+ * String utility
+ */
+static int
+opng_strcasecmp(const char *str1, const char *str2)
 {
-    char *endptr;
+    int ch1, ch2;
+
+    /* Perform a case-insensitive string comparison. */
+    for ( ; ; )
+    {
+        ch1 = tolower(*str1++);
+        ch2 = tolower(*str2++);
+        if (ch1 != ch2)
+            return ch1 - ch2;
+        if (ch1 == 0)
+            return 0;
+    }
+    /* FIXME: This function is not MBCS-aware. */
+}
+
+
+/*
+ * String utility
+ */
+static char *
+opng_strltrim(const char *str)
+{
+    /* Skip the leading whitespace characters. */
+    while (isspace(*str))
+        ++str;
+    return (char *)str;
+}
+
+
+/*
+ * String utility
+ */
+static char *
+opng_strtail(const char *str, size_t num)
+{
+    size_t len;
+
+    /* Return up to num rightmost characters. */
+    len = strlen(str);
+    if (len <= num)
+        return (char *)str;
+    return (char *)str + len - num;
+}
+
+
+/*
+ * String conversion utility
+ */
+static int
+opng_str2ulong_base10(unsigned long *out_val, const char *in_str,
+                      int allow_multiplier)
+{
+    const char *begin_ptr;
+    char *end_ptr;
+    unsigned long multiplier;
 
     /* Extract the value from the string. */
-    *value = strtol(str, &endptr, 10);
-    if (endptr == NULL || endptr == str)
+    /* Do not allow the minus sign, not even for -0. */
+    begin_ptr = end_ptr = opng_strltrim(in_str);
+    if (*begin_ptr >= '0' && *begin_ptr <= '9')
+        *out_val = strtoul(begin_ptr, &end_ptr, 10);
+    if (begin_ptr == end_ptr)
     {
         errno = EINVAL;  /* matching failure */
+        *out_val = 0;
         return -1;
     }
 
-    /* Check for the 'kilo' suffix. */
-    if (*endptr == 'k' || *endptr == 'K')
+    if (allow_multiplier)
     {
-        ++endptr;
-        if (*value > LONG_MAX / 1024)
+        /* Check for the following SI suffixes:
+         *   'K' or 'k': kibi (1024)
+         *   'M':        mebi (1024 * 1024)
+         *   'G':        gibi (1024 * 1024 * 1024)
+         */
+        if (*end_ptr == 'k' || *end_ptr == 'K')
         {
-            errno = ERANGE;  /* overflow */
-            *value = LONG_MAX;
+            ++end_ptr;
+            multiplier = 1024UL;
         }
-        else if (*value < LONG_MIN / 1024)
+        else if (*end_ptr == 'M')
         {
-            errno = ERANGE;  /* overflow */
-            *value = LONG_MIN;
+            ++end_ptr;
+            multiplier = 1024UL * 1024UL;
+        }
+        else if (*end_ptr == 'G')
+        {
+            ++end_ptr;
+            multiplier = 1024UL * 1024UL * 1024UL;
         }
         else
-            *value *= 1024;
+            multiplier = 1;
+        if (multiplier > 1)
+        {
+            if (*out_val > ULONG_MAX / multiplier)
+            {
+                errno = ERANGE;  /* overflow */
+                *out_val = ULONG_MAX;
+            }
+            else
+                *out_val *= multiplier;
+        }
     }
 
     /* Check for trailing garbage. */
-    while (isspace(*endptr))
-        ++endptr;  /* skip whitespace */
-    if (*endptr != 0)
+    if (*opng_strltrim(end_ptr) != 0)
     {
         errno = EINVAL;  /* garbage in input */
         return -1;
     }
-
     return 0;
 }
 
 
-/** Command line error handling **/
-static void
-err_option(const char *opt_desc, const char *opt_arg)
+/*
+ * String conversion utility
+ */
+static int
+opng_rangeset2bitset(bitset_t *out_val, const char *in_str)
 {
-    /* Issue an error regarding the incorrect use of the option. */
-    if (opt_arg != NULL && opt_arg[0] != 0)
-        error("Invalid %s: %s", opt_desc, opt_arg);
-    else
-        error("Missing %s", opt_desc);
+    size_t end_idx;
+
+    /* Extract the bitset value from the rangeset string. */
+    *out_val = rangeset_string_to_bitset(in_str, &end_idx);
+    if (end_idx == 0 || *opng_strltrim(in_str + end_idx) != 0)
+    {
+        errno = EINVAL;
+        return -1;
+    }
+    return 0;
 }
 
 
-/** Command line parsing **/
-static int
-scan_option(char *str, char opt_buf[], size_t opt_buf_size, char **opt_arg_ptr)
+/*
+ * Command line utility
+ */
+static void
+err_option_arg(const char *opt, const char *opt_arg)
 {
-    char *ptr;
+    /* Issue an error regarding the incorrect value of the option argument. */
+    if (opt_arg == NULL || *opng_strltrim(opt_arg) == 0)
+        error("Missing argument for option %s", opt);
+    else
+        error("Invalid argument for option %s: %s", opt, opt_arg);
+}
+
+
+/*
+ * Command line utility
+ */
+static int
+check_num_option(const char *opt, const char *opt_arg,
+                 int lowest, int highest)
+{
+    unsigned long value;
+
+    /* Extract the numeric value from the option argument. */
+    if (opng_str2ulong_base10(&value, opt_arg, 0) != 0 ||
+            value > INT_MAX || (int)value < lowest || (int)value > highest)
+        err_option_arg(opt, opt_arg);
+    return (int)value;
+}
+
+
+/*
+ * Command line utility
+ */
+static int
+check_power2_option(const char *opt, const char *opt_arg,
+                    int lowest, int highest)
+{
+    unsigned long value;
+    int result;
+
+    /* Extract the exact log2 of the numeric value from the option argument. */
+    /* Allow the 'k', 'M', 'G' suffixes. */
+    if (opng_str2ulong_base10(&value, opt_arg, 1) == 0)
+    {
+        if (lowest < 0)
+            lowest = 0;
+        if (highest > (int)(CHAR_BIT * sizeof(long) - 2))
+            highest = (int)(CHAR_BIT * sizeof(long) - 2);
+        for (result = lowest; result <= highest; ++result)
+        {
+            if ((1UL << result) == value)
+                return result;
+        }
+    }
+    err_option_arg(opt, opt_arg);
+    return -1;
+}
+
+
+/*
+ * Command line utility
+ */
+static bitset_t
+check_rangeset_option(const char *opt, const char *opt_arg,
+                      bitset_t result_mask)
+{
+    bitset_t result;
+
+    /* Extract the rangeset from the option argument. */
+    if (opng_rangeset2bitset(&result, opt_arg) == 0)
+        result &= result_mask;
+    else
+        result = BITSET_EMPTY;
+    if (result == BITSET_EMPTY)
+        err_option_arg(opt, opt_arg);
+    return result;
+}
+
+
+/*
+ * Command line parsing
+ */
+static int
+scan_option(const char *str,
+            char opt_buf[], size_t opt_buf_size, char **opt_arg_ptr)
+{
+    const char *ptr;
     unsigned int opt_len;
 
     /* Check if arg is an "-option". */
@@ -284,22 +452,24 @@ scan_option(char *str, char opt_buf[], size_t opt_buf_size, char **opt_arg_ptr)
         ++ptr;
     else while (isspace(*ptr))  /* "-option arg" */
         ++ptr;
-    *opt_arg_ptr = (*ptr != 0) ? ptr : NULL;
+    *opt_arg_ptr = (*ptr != 0) ? (char *)ptr : NULL;
     return 1;
 }
 
 
-/** Command line parsing **/
+/*
+ * Command line parsing
+ */
 static void
 parse_args(int argc, char *argv[])
 {
     char opt[16];
+    size_t opt_len;
     char *arg, *xopt;
     unsigned int file_count;
-    int stop_switch, i;
-    int val;
-    long lval;
+    int stop_switch;
     bitset_t set;
+    int val, i;
 
     /* Initialize. */
     memset(&options, 0, sizeof(options));
@@ -317,85 +487,109 @@ parse_args(int argc, char *argv[])
             ++file_count;
             continue;  /* leave file names for process_files() */
         }
+        opt_len = strlen(opt);
 
         /* Prevent process_files() from seeing this arg. */
         argv[i] = NULL;
 
         /* Check the simple options (without option arguments). */
-        if (strcmp(opt, "-") == 0)  /* "--" */
+        if (strcmp("-", opt) == 0)
         {
+            /* -- */
             stop_switch = 1;
         }
-        else if (strcmp(opt, "?") == 0 ||
-                 string_prefix_min_cmp("help", opt, 1) == 0)
+        else if (strcmp("?", opt) == 0 ||
+                 strncmp("help", opt, opt_len) == 0)
         {
+            /* -? | -h | ... | -help */
             options.help = 1;
         }
-        else if (string_prefix_min_cmp("fix", opt, 2) == 0)
+        else if (strncmp("fix", opt, opt_len) == 0 && opt_len >= 2)
         {
+            /* -fi | -fix */
             options.fix = 1;
         }
-        else if (string_prefix_min_cmp("force", opt, 2) == 0)
+        else if (strncmp("force", opt, opt_len) == 0 && opt_len >= 2)
         {
+            /* -fo | ... | -force */
             options.force = 1;
         }
-        else if (string_prefix_min_cmp("full", opt, 2) == 0)
+        else if (strncmp("full", opt, opt_len) == 0 && opt_len >= 2)
         {
+            /* -fu | ... | -full */
             options.full = 1;
         }
-        else if (string_prefix_min_cmp("keep", opt, 1) == 0)
+        else if (strncmp("keep", opt, opt_len) == 0)
         {
+            /* -k | ... | -keep */
             options.keep = 1;
         }
-        else if (strcmp(opt, "nb") == 0)
+        else if (strcmp("nb", opt) == 0)
         {
+            /* -nb */
             options.nb = 1;
         }
-        else if (strcmp(opt, "nc") == 0)
+        else if (strcmp("nc", opt) == 0)
         {
+            /* -nc */
             options.nc = 1;
         }
-#if 0
-        else if (strcmp(opt, "nm") == 0)
+        else if (strcmp("nm", opt) == 0)
         {
-            options.nm = 1;
+            /* -nm */
+            /* options.nm = 1; */
+            error("Metadata optimization is not implemented");
         }
-#endif
-        else if (strcmp(opt, "np") == 0)
+        else if (strcmp("np", opt) == 0)
         {
+            /* -np */
             options.np = 1;
         }
-        else if (strcmp(opt, "nz") == 0)
+        else if (strcmp("nx", opt) == 0)
         {
+            /* -nx */
+            options.nb = options.nc = options.np = 1;
+            /* options.nm = 1; */
+        }
+        else if (strcmp("nz", opt) == 0)
+        {
+            /* -nz */
             options.nz = 1;
         }
-        else if (string_prefix_min_cmp("preserve", opt, 1) == 0)
+        else if (strncmp("preserve", opt, opt_len) == 0)
         {
+            /* -p | ... | -preserve */
             options.preserve = 1;
         }
-        else if (string_prefix_min_cmp("quiet", opt, 1) == 0)
+        else if (strncmp("quiet", opt, opt_len) == 0)
         {
+            /* -q | ... | -quiet */
             options.quiet = 1;
         }
-        else if (string_prefix_min_cmp("simulate", opt, 2) == 0)
+        else if (strncmp("simulate", opt, opt_len) == 0 && opt_len >= 2)
         {
+            /* -si | ... | -simulate */
             options.simulate = 1;
         }
-        else if (string_prefix_min_cmp("snip", opt, 2) == 0)
+        else if (strncmp("snip", opt, opt_len) == 0 && opt_len >= 2)
         {
+            /* -sn | ... | -snip */
             options.snip = 1;
         }
-        else if (strcmp(opt, "v") == 0)
+        else if (strcmp("v", opt) == 0)
         {
+            /* -v */
             options.verbose = 1;
             options.version = 1;
         }
-        else if (string_prefix_min_cmp("verbose", opt, 4) == 0)
+        else if (strncmp("verbose", opt, opt_len) == 0 && opt_len >= 4)
         {
+            /* -verb | ... | -verbose */
             options.verbose = 1;
         }
-        else if (string_prefix_min_cmp("version", opt, 4) == 0)
+        else if (strncmp("version", opt, opt_len) == 0 && opt_len >= 4)
         {
+            /* -vers | ... | -version */
             options.version = 1;
         }
         else  /* possibly an option with an argument */
@@ -416,104 +610,110 @@ parse_args(int argc, char *argv[])
         /* Check the options that have option arguments. */
         if (xopt == NULL)
         {
-            /* Do nothing, an option without argument is already recognized. */
+            /* An option without argument has already been recognized. */
         }
-        else if (strcmp(opt, "o") == 0)
+        else if (strcmp("o", opt) == 0)
         {
-            if (str2long(xopt, &lval) != 0 || lval < 0 || lval > 99)
-                err_option("optimization level", xopt);
-            val = (int)lval;
+            /* -o NUM */
+            val = check_num_option("-o", xopt, 0, INT_MAX);
             if (options.optim_level < 0)
                 options.optim_level = val;
             else if (options.optim_level != val)
                 error("Multiple optimization levels are not permitted");
         }
-        else if (strcmp(opt, "i") == 0)
+        else if (strcmp("i", opt) == 0)
         {
-            if (str2long(xopt, &lval) != 0 || lval < 0 || lval > 1)
-                err_option("interlace type", xopt);
-            val = (int)lval;
+            /* -i NUM */
+            val = check_num_option("-i", xopt, 0, 1);
             if (options.interlace < 0)
-                options.interlace = (int)val;
-            else if (options.interlace != (int)val)
+                options.interlace = val;
+            else if (options.interlace != val)
                 error("Multiple interlace types are not permitted");
         }
-        else if (strcmp(opt, "b") == 0)
+        else if (strcmp("b", opt) == 0)
         {
+            /* -b NUM */
             /* options.bit_depth = ... */
             error("Selection of bit depth is not implemented");
         }
-        else if (strcmp(opt, "c") == 0)
+        else if (strcmp("c", opt) == 0)
         {
+            /* -c NUM */
             /* options.color_type = ... */
             error("Selection of color type is not implemented");
         }
-        else if (strcmp(opt, "f") == 0)
+        else if (strcmp("f", opt) == 0)
         {
-            if (bitset_parse(xopt, &set) != 0)
-                err_option("filter(s)", xopt);
+            /* -f SET */
+            set = check_rangeset_option("-f", xopt, OPNG_FILTER_SET_MASK);
             options.filter_set |= set;
         }
-        else if (strcmp(opt, "zc") == 0)
+        else if (strcmp("zc", opt) == 0)
         {
-            if (bitset_parse(xopt, &set) != 0)
-                err_option("zlib compression level(s)", xopt);
+            /* -zc SET */
+            set = check_rangeset_option("-zc", xopt, OPNG_COMPR_LEVEL_SET_MASK);
             options.compr_level_set |= set;
         }
-        else if (strcmp(opt, "zm") == 0)
+        else if (strcmp("zm", opt) == 0)
         {
-            if (bitset_parse(xopt, &set) != 0)
-                err_option("zlib memory level(s)", xopt);
+            /* -zm SET */
+            set = check_rangeset_option("-zm", xopt, OPNG_MEM_LEVEL_SET_MASK);
             options.mem_level_set |= set;
         }
-        else if (strcmp(opt, "zs") == 0)
+        else if (strcmp("zs", opt) == 0)
         {
-            if (bitset_parse(xopt, &set) != 0)
-                err_option("zlib compression strategy", xopt);
+            /* -zs SET */
+            set = check_rangeset_option("-zs", xopt, OPNG_STRATEGY_SET_MASK);
             options.strategy_set |= set;
         }
-        else if (strcmp(opt, "zw") == 0)
+        else if (strcmp("zw", opt) == 0)
         {
-            if (str2long(xopt, &lval) != 0)
-                lval = 0;
-            for (val = 15; val >= 8; --val)
-                if ((1L << val) == lval)
-                    break;
-            if (val < 8)
-                err_option("zlib window size", xopt);
+            /* -zw NUM */
+            val = check_power2_option("-zw", xopt, 8, 15);
             if (options.window_bits == 0)
                 options.window_bits = val;
             else if (options.window_bits != val)
                 error("Multiple window sizes are not permitted");
         }
-        else if (string_prefix_min_cmp("out", opt, 2) == 0)
+        else if (strncmp("out", opt, opt_len) == 0 && opt_len >= 2)
         {
+            /* -ou PATH | -out PATH */
             if (options.out_name != NULL)
-                error("Duplicate output file name");
+                error("Multiple output file names are not permitted");
             if (xopt[0] == 0)
-                err_option("output file name", xopt);
+                err_option_arg("-out", NULL);
             options.out_name = xopt;
         }
-        else if (string_prefix_min_cmp("dir", opt, 1) == 0)
+        else if (strncmp("dir", opt, opt_len) == 0)
         {
+            /* -d PATH | ... | -dir PATH */
             if (options.dir_name != NULL)
-                error("Duplicate output dir name");
+                error("Multiple output dir names are not permitted");
             if (xopt[0] == 0)
-                err_option("output dir name", xopt);
+                err_option_arg("-dir", NULL);
             options.dir_name = xopt;
         }
-        else if (string_prefix_min_cmp("log", opt, 1) == 0)
+        else if (strncmp("log", opt, opt_len) == 0)
         {
+            /* -l PATH | ... | -log PATH */
             if (options.log_name != NULL)
-                error("Duplicate log file name");
+                error("Multiple log file names are not permitted");
             if (xopt[0] == 0)
-                err_option("log file name", xopt);
+                err_option_arg("-log", NULL);
             options.log_name = xopt;
         }
-        else if (string_prefix_min_cmp("jobs", opt, 1) == 0)
+        else if (strncmp("jobs", opt, opt_len) == 0)
         {
+            /* -j NUM | ... | -jobs NUM */
             error("Parallel processing is not implemented");
         }
+        else if (strcmp("erase", opt) == 0 ||
+                 strcmp("strip", opt) == 0 ||
+                 strcmp("protect", opt) == 0)
+        {
+            /* -erase DATA | -strip DATA | -protect DATA */
+            error("Lossy operations are not currently supported");
+        }
         else
         {
             error("Unrecognized option: %s", arg);
@@ -524,25 +724,23 @@ parse_args(int argc, char *argv[])
     if (options.out_name != NULL)
     {
         if (file_count > 1)
-            error("-out requires one input file");
+            error("The option -out requires one input file");
         if (options.dir_name != NULL)
-            error("-out and -dir are mutually exclusive");
+            error("The options -out and -dir are mutually exclusive");
     }
     if (options.log_name != NULL)
     {
-        if (string_suffix_case_cmp(options.log_name, ".log") != 0)
-            error("To prevent accidental data corruption,"
-                  " the log file name must end with \".log\"");
+        if (opng_strcasecmp(".log", opng_strtail(options.log_name, 4)) != 0)
+            error("To prevent accidental data corruption, "
+                  "the log file name must end with \".log\"");
     }
-    if (options.optim_level == 0)
-        options.nz = 1;
-    if (options.nz)
-        options.nb = options.nc = options.np = 1;
     operation = (options.help || file_count == 0) ? OP_HELP : OP_RUN;
 }
 
 
-/** Initialization **/
+/*
+ * Initialization
+ */
 static void
 app_init(void)
 {
@@ -562,7 +760,9 @@ app_init(void)
 }
 
 
-/** Finalization **/
+/*
+ * Finalization
+ */
 static void
 app_finish(void)
 {
@@ -574,7 +774,9 @@ app_finish(void)
 }
 
 
-/** Application-defined printf callback **/
+/*
+ * Application-defined printf callback
+ */
 static void
 app_printf(const char *fmt, ...)
 {
@@ -599,7 +801,9 @@ app_printf(const char *fmt, ...)
 }
 
 
-/** Application-defined control print callback **/
+/*
+ * Application-defined control print callback
+ */
 static void
 app_print_cntrl(int cntrl_code)
 {
@@ -642,13 +846,15 @@ app_print_cntrl(int cntrl_code)
     }
 
     if (con_file != NULL)
-        fprintf(con_file, con_str);
+        fputs(con_str, con_file);
     if (log_file != NULL)
-        fprintf(log_file, log_str);
+        fputs(log_str, log_file);
 }
 
 
-/** Application-defined progress update callback **/
+/*
+ * Application-defined progress update callback
+ */
 static void
 app_progress(unsigned long current_step, unsigned long total_steps)
 {
@@ -664,7 +870,9 @@ app_progress(unsigned long current_step, unsigned long total_steps)
 }
 
 
-/** File list processing **/
+/*
+ * File list processing
+ */
 static int
 process_files(int argc, char *argv[])
 {
@@ -698,7 +906,9 @@ process_files(int argc, char *argv[])
 }
 
 
-/** main **/
+/*
+ * main
+ */
 int
 main(int argc, char *argv[])
 {
