@@ -1,12 +1,12 @@
-/**
- ** optipng.h
- ** OptiPNG programming interface.
- **
- ** Copyright (C) 2001-2009 Cosmin Truta.
- **
- ** This software is distributed under the zlib license.
- ** Please see the attached LICENSE for more information.
- **/
+/*
+ * optipng.h
+ * OptiPNG programming interface.
+ *
+ * Copyright (C) 2001-2010 Cosmin Truta.
+ *
+ * This software is distributed under the zlib license.
+ * Please see the attached LICENSE for more information.
+ */
 
 
 #ifndef OPTIPNG_H
@@ -21,7 +21,7 @@ extern "C" {
 
 
 /*
- * Program options (e.g. extracted from the command line)
+ * User options (e.g. extracted from the command line)
  */
 struct opng_options
 {
@@ -51,7 +51,7 @@ struct opng_options
 
 
 /*
- * Program UI callbacks
+ * User interface callbacks
  */
 struct opng_ui
 {
@@ -81,6 +81,30 @@ int opng_optimize(const char *infile_name);
 int opng_finalize(void);
 
 
+/*
+ * Encoder limits and default values
+ */
+#define OPNG_OPTIM_LEVEL_DEFAULT    2
+#define OPNG_OPTIM_LEVEL_MIN        0
+#define OPNG_OPTIM_LEVEL_MAX        7
+
+#define OPNG_COMPR_LEVEL_MIN        1
+#define OPNG_COMPR_LEVEL_MAX        9
+#define OPNG_COMPR_LEVEL_SET_MASK   ((1 << (9+1)) - (1 << 1))  /* 0x03fe */
+
+#define OPNG_MEM_LEVEL_MIN          1
+#define OPNG_MEM_LEVEL_MAX          9
+#define OPNG_MEM_LEVEL_SET_MASK     ((1 << (9+1)) - (1 << 1))  /* 0x03fe */
+
+#define OPNG_STRATEGY_MIN           0
+#define OPNG_STRATEGY_MAX           3
+#define OPNG_STRATEGY_SET_MASK      ((1 << (3+1)) - (1 << 0))  /* 0x000f */
+
+#define OPNG_FILTER_MIN             0
+#define OPNG_FILTER_MAX             5
+#define OPNG_FILTER_SET_MASK        ((1 << (5+1)) - (1 << 0))  /* 0x003f */
+
+
 #ifdef __cplusplus
 }  /* extern "C" */
 #endif
